mod race;
use race::Race;

use iced::widget::{button, column, combo_box, row, text, Column, ComboBox, Text};
use iced::{Alignment, Padding};

/// Represents the state of our application and handles
/// message matching and updating the state accordingly.
struct State {
    races: combo_box::State<Race>,
    selected_race: Option<Race>,
    text: String,
    counter_value: i32,
}

// Default is expected by iced::run.
// We need to manually implement this since combo_box is not derive(Default)
impl Default for State {
    fn default() -> Self {
        Self::new()
    }
}

// Represents the messages that can be sent to the "component"
#[derive(Debug, Clone, Copy)]
enum Command {
    /// Increment the counter
    Increment,
    /// Decrement the counter
    Decrement,
    /// User selected an option in the combo box
    Selected(Race),
    /// User hovered over an option in the combo box
    OptionHovered(Race),
    /// User closed the combo box
    Closed,
}

impl State {
    // Constructor for the state, nothing special here.
    fn new() -> Self {
        Self {
            races: combo_box::State::new(Race::ALL.to_vec()),
            selected_race: None,
            text: String::from("Hello, World!"),
            counter_value: 10,
        }
    }

    // On each update, we recieve a command and update the state accordingly.
    fn update(&mut self, command: Command) {
        match command {
            Command::Increment => self.counter_value += 1,
            Command::Decrement => self.counter_value -= 1,
            Command::Selected(race) => self.selected_race = Some(race),
            Command::OptionHovered(race) => println!("Hovered over {}.", race.to_string()),
            Command::Closed => self.selected_race = None,
        }
    }

    // Called every frame to render the user interface.
    // Changes to the state are reflected here.
    fn view(&self) -> Column<Command> {
        // Our combo box
        let combo_box: ComboBox<Race, Command> = combo_box::ComboBox::new(
            &self.races,
            "Pick a race",
            self.selected_race.as_ref(),
            Command::Selected,
        )
        .on_option_hovered(Command::OptionHovered)
        .on_close(Command::Closed)
        .width(250);

        // Counter value
        let counter_value: Text = Text::new(self.counter_value.to_string())
            .size(50)
            .width(50)
            .horizontal_alignment(iced::alignment::Horizontal::Center);

        // Row containig the buttons and the counter value
        let counter_row = row![
            button("Increment").on_press(Command::Increment),
            counter_value,
            button("Decrement").on_press(Command::Decrement),
        ]
        .spacing(10)
        .align_items(Alignment::Start);

        // Finally return a combination of the components
        column![counter_row, combo_box, text(&self.text)]
            .padding(Padding::new(self.counter_value as f32))
            .align_items(Alignment::Start)
            .spacing(10)
            .width(iced::Length::Fill)
    }
}

fn main() {
    // Iced::run is an entry point for the application.
    // It may fail so we match on the result.
    match iced::run("Test Example", State::update, State::view) {
        Ok(_) => {}
        Err(error) => eprintln!("Error: {}", error),
    }
}
